@@ -8,12 +8,11 @@
 #include "mapcssparser_p.h"
 #include "logging.h"
 
+#include "mapcssloader.h"
 #include "mapcssparser_impl.h"
 #include "mapcssdeclaration_p.h"
-#include "mapcssrule_p.h"
 #include "mapcssscanner.h"
 #include "mapcssstyle.h"
-#include "mapcssstyle_p.h"
 
 #include <QDebug>
 #include <QFile>
@@ -22,7 +21,8 @@
 
 #include <cstring>
 
-char* unquoteString(const char *str)
+template <char Q>
+[[nodiscard]] static char* unquoteString(const char *str)
 {
     const auto size = strlen(str) - 2;
     if (size <= 0) {
@@ -36,7 +36,7 @@ char* unquoteString(const char *str)
             ++it;
             switch (*it) {
                 case '\\':
-                case '"':
+                case Q:
                     *outIt = *it; break;
                 case 'n':
                     *outIt = '\n'; break;
@@ -53,6 +53,16 @@ char* unquoteString(const char *str)
     return out;
 }
 
+[[nodiscard]] char* unquoteSingleQuotedString(const char *str)
+{
+    return unquoteString<'\''>(str);
+}
+
+[[nodiscard]] char* unquoteDoubleQuotedString(const char *str)
+{
+    return unquoteString<'"'>(str);
+}
+
 using namespace KOSMIndoorMap;
 
 MapCSSParser::MapCSSParser()
@@ -63,13 +73,18 @@ MapCSSParser::MapCSSParser()
 MapCSSParser::~MapCSSParser() = default;
 
 bool MapCSSParser::hasError() const
+{
+    return d->m_error != MapCSSParser::NoError;
+}
+
+MapCSSParser::Error MapCSSParser::error() const
 {
     return d->m_error;
 }
 
-QString MapCSSParser::fileName() const
+QUrl MapCSSParser::url() const
 {
-    return d->m_currentFileName;
+    return d->m_currentUrl;
 }
 
 QString MapCSSParser::errorMessage() const
@@ -78,15 +93,18 @@ QString MapCSSParser::errorMessage() const
         return {};
     }
 
-    return d->m_errorMsg + QLatin1String(": ") + fileName() + QLatin1Char(':') + QString::number(d->m_line) + QLatin1Char(':') + QString::number(d->m_column);
+    return d->m_errorMsg + QLatin1String(": ") + url().toString() + QLatin1Char(':') + QString::number(d->m_line) + QLatin1Char(':') + QString::number(d->m_column);
 }
 
 MapCSSStyle MapCSSParser::parse(const QString &fileName)
 {
-    d->m_error = true;
+    return parse(MapCSSLoader::resolve(fileName));
+}
 
+MapCSSStyle MapCSSParser::parse(const QUrl &url)
+{
     MapCSSStyle style;
-    d->parse(&style, fileName, {});
+    d->parse(&style, url, {});
     if (d->m_error) {
         return MapCSSStyle();
     }
@@ -94,21 +112,27 @@ MapCSSStyle MapCSSParser::parse(const QString &fileName)
     return style;
 }
 
-void MapCSSParserPrivate::parse(MapCSSStyle *style, const QString &fileName, ClassSelectorKey importClass)
+void MapCSSParserPrivate::parse(MapCSSStyle *style, const QUrl &url, ClassSelectorKey importClass)
 {
+    const auto fileName = MapCSSLoader::toLocalFile(url);
     QFile f(fileName);
     if (!f.open(QFile::ReadOnly)) {
         qCWarning(Log) << f.fileName() << f.errorString();
-        m_error = true;
+        if (!QFile::exists(fileName)) {
+            m_error = MapCSSParser::FileNotFoundError;
+            m_currentUrl = url;
+        } else {
+            m_error = MapCSSParser::FileIOError;
+        }
         m_errorMsg = f.errorString();
         return;
     }
-    m_currentFileName = fileName;
+    m_currentUrl = url;
     m_currentStyle = style;
     m_importClass = importClass;
 
     yyscan_t scanner;
-    if (yylex_init(&scanner)) {
+    if (yylex_init_extra(this, &scanner)) {
         return;
     }
     const auto lexerCleanup = qScopeGuard([&scanner]{ yylex_destroy(scanner); });
@@ -116,63 +140,14 @@ void MapCSSParserPrivate::parse(MapCSSStyle *style, const QString &fileName, Cla
     const auto b = f.readAll();
     YY_BUFFER_STATE state;
     state = yy_scan_string(b.constData(), scanner);
+    m_error = MapCSSParser::SyntaxError;
     if (yyparse(this, scanner)) {
-        m_error = true;
         return;
     }
 
     yy_delete_buffer(state, scanner);
 
-    m_error = false;
+    m_error = MapCSSParser::NoError;
     m_currentStyle = nullptr;
     m_importClass = {};
 }
-
-bool MapCSSParserPrivate::addImport(char* fileName, ClassSelectorKey importClass)
-{
-    auto cssFile = QString::fromUtf8(fileName);
-    free(fileName);
-
-    if (QFileInfo(cssFile).isRelative()) {
-        cssFile = QFileInfo(m_currentFileName).absolutePath() + QLatin1Char('/') + cssFile;
-    }
-
-    MapCSSParser p;
-    p.d->parse(m_currentStyle, cssFile, importClass);
-    if (p.hasError()) {
-        m_error = p.d->m_error;
-        m_errorMsg = p.errorMessage();
-    }
-    return !p.hasError();
-}
-
-void MapCSSParserPrivate::addRule(MapCSSRule *rule)
-{
-    if (!m_importClass.isNull()) {
-        auto decl = new MapCSSDeclaration(MapCSSDeclaration::ClassDeclaration);
-        decl->setClassSelectorKey(m_importClass);
-        rule->addDeclaration(decl);
-    }
-    MapCSSStylePrivate::get(m_currentStyle)->m_rules.push_back(std::unique_ptr<MapCSSRule>(rule));
-}
-
-void MapCSSParserPrivate::setError(const QString &msg, int line, int column)
-{
-    m_error = true;
-    m_errorMsg = msg;
-    m_line = line;
-    m_column = column;
-}
-
-ClassSelectorKey MapCSSParserPrivate::makeClassSelector(const char *str, std::size_t len) const
-{
-    return MapCSSStylePrivate::get(m_currentStyle)->m_classSelectorRegistry.makeKey(str, len, OSM::StringMemory::Transient);
-}
-
-LayerSelectorKey MapCSSParserPrivate::makeLayerSelector(const char *str, std::size_t len) const
-{
-    if (!str || std::strcmp(str, "default") == 0) {
-        return {};
-    }
-    return MapCSSStylePrivate::get(m_currentStyle)->m_layerSelectorRegistry.makeKey(str, len, OSM::StringMemory::Transient);
-}
